@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,29 +20,37 @@
 
 
 # Imports
-from bip_utils.bech32 import AtomBech32Encoder
-from bip_utils.utils import CryptoUtils, KeyUtils
+from typing import Any, Union
+from bip_utils.addr.iaddr_encoder import IAddrEncoder
+from bip_utils.addr.utils import AddrUtils
+from bip_utils.bech32 import Bech32Encoder
+from bip_utils.ecc import IPublicKey
+from bip_utils.utils.misc import CryptoUtils
 
 
-class AtomAddr:
+class AtomAddr(IAddrEncoder):
     """ Atom address class. It allows the Atom address generation. """
 
     @staticmethod
-    def ToAddress(pub_key_bytes: bytes,
-                  hrp: str) -> str:
+    def EncodeKey(pub_key: Union[bytes, IPublicKey],
+                  **kwargs: Any) -> str:
         """ Get address in Atom format.
 
         Args:
-            pub_key_bytes (bytes): Public key bytes
-            hrp (str)            : HRP
+            pub_key (bytes or IPublicKey): Public key bytes or object
+
+        Other Parameters:
+            hrp (str): HRP
 
         Returns:
             str: Address string
 
         Raises:
-            ValueError: If key is not a public compressed key
+            ValueError: If the public key is not valid
+            TypeError: If the public key is not secp256k1
         """
-        if not KeyUtils.IsPublicCompressed(pub_key_bytes):
-            raise ValueError("Public compressed key is required for Atom address")
+        hrp = kwargs["hrp"]
 
-        return AtomBech32Encoder.Encode(hrp, CryptoUtils.Hash160(pub_key_bytes))
+        pub_key_obj = AddrUtils.ValidateAndGetSecp256k1Key(pub_key)
+
+        return Bech32Encoder.Encode(hrp, CryptoUtils.Hash160(pub_key_obj.RawCompressed().ToBytes()))
