@@ -20,49 +20,53 @@
 
 
 # Imports
-from bip_utils.bech32 import AvaxChainTypes, AvaxBech32Encoder
-from bip_utils.utils import CryptoUtils, KeyUtils
+from typing import Any, Union
+from bip_utils.addr.atom_addr import AtomAddr
+from bip_utils.addr.iaddr_encoder import IAddrEncoder
+from bip_utils.coin_conf import AvaxPChainConf, AvaxXChainConf
+from bip_utils.ecc import IPublicKey
 
 
-class AvaxXChainAddr:
-    """ Avax X-Chain address class. It allows the Avax X-Chain address generation. """
-
-    @staticmethod
-    def ToAddress(pub_key_bytes: bytes) -> str:
-        """ Get address in Atom format.
-
-        Args:
-            pub_key_bytes (bytes) : Public key bytes
-
-        Returns:
-            str: Address string
-
-        Raises:
-            ValueError: If key is not a public compressed key
-        """
-        if not KeyUtils.IsPublicCompressed(pub_key_bytes):
-            raise ValueError("Public compressed key is required for Avax address")
-
-        return AvaxBech32Encoder.Encode(CryptoUtils.Hash160(pub_key_bytes), AvaxChainTypes.AVAX_X_CHAIN)
-
-
-class AvaxPChainAddr:
+class AvaxPChainAddr(IAddrEncoder):
     """ Avax P-Chain address class. It allows the Avax P-Chain address generation. """
 
     @staticmethod
-    def ToAddress(pub_key_bytes: bytes) -> str:
+    def EncodeKey(pub_key: Union[bytes, IPublicKey],
+                  **kwargs: Any) -> str:
         """ Get address in Atom format.
 
         Args:
-            pub_key_bytes (bytes) : Public key bytes
+            pub_key (bytes or IPublicKey): Public key bytes or object
+            **kwargs: Not used
 
         Returns:
             str: Address string
 
         Raises:
-            ValueError: If key is not a public compressed key
+            ValueError: If the public key is not valid
+            TypeError: If the public key is not secp256k1
         """
-        if not KeyUtils.IsPublicCompressed(pub_key_bytes):
-            raise ValueError("Public compressed key is required for Avax address")
+        return AvaxPChainConf.ADDR_PREFIX + AtomAddr.EncodeKey(pub_key,
+                                                               hrp=AvaxPChainConf.ADDR_HRP)
 
-        return AvaxBech32Encoder.Encode(CryptoUtils.Hash160(pub_key_bytes), AvaxChainTypes.AVAX_P_CHAIN)
+
+class AvaxXChainAddr(IAddrEncoder):
+    """ Avax X-Chain address class. It allows the Avax X-Chain address generation. """
+
+    @staticmethod
+    def EncodeKey(pub_key: Union[bytes, IPublicKey],
+                  **kwargs: Any) -> str:
+        """ Get address in Atom format.
+
+        Args:
+            pub_key (bytes or IPublicKey): Public key bytes or object
+
+        Returns:
+            str: Address string
+
+        Raises:
+            ValueError: If the public key is not valid
+            TypeError: If the public key is not secp256k1
+        """
+        return AvaxXChainConf.ADDR_PREFIX + AtomAddr.EncodeKey(pub_key,
+                                                               hrp=AvaxXChainConf.ADDR_HRP)
