@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,15 +20,17 @@
 
 
 # Imports
-import sha3
-from bip_utils.utils import AlgoUtils, KeyUtils
+from typing import Any, Union
+from bip_utils.addr.iaddr_encoder import IAddrEncoder
+from bip_utils.addr.utils import AddrUtils
+from bip_utils.coin_conf import EthereumConf
+from bip_utils.ecc import IPublicKey
+from bip_utils.utils.misc import ConvUtils, CryptoUtils
 
 
 class EthAddrConst:
     """ Class container for Ethereum address constants. """
 
-    # Prefix
-    PREFIX: str = "0x"
     # Start byte
     START_BYTE: int = 24
 
@@ -48,31 +50,35 @@ class EthAddrUtils:
         """
 
         # Compute address digest
-        addr_digest = sha3.keccak_256(AlgoUtils.Encode(addr)).hexdigest()
+        addr_hex_digest = ConvUtils.BytesToHexString(CryptoUtils.Kekkak256(addr))
         # Encode it
-        enc_addr = [c.upper() if (int(addr_digest[i], 16) >= 8) else c.lower() for i, c in enumerate(addr)]
+        enc_addr = [c.upper() if (int(addr_hex_digest[i], 16) >= 8) else c.lower() for i, c in enumerate(addr)]
 
         return "".join(enc_addr)
 
 
-class EthAddr:
+class EthAddr(IAddrEncoder):
     """ Ethereum address class. It allows the Ethereum address generation. """
 
     @staticmethod
-    def ToAddress(pub_key_bytes: bytes) -> str:
+    def EncodeKey(pub_key: Union[bytes, IPublicKey],
+                  **kwargs: Any) -> str:
         """ Get address in Ethereum format.
 
         Args:
-            pub_key_bytes (bytes): Public key bytes
+            pub_key (bytes or IPublicKey): Public key bytes or object
+            **kwargs: Not used
 
         Returns:
             str: Address string
 
         Raised:
-            ValueError: If the key is not a public uncompressed key
+            ValueError: If the public key is not valid
+            TypeError: If the public key is not secp256k1
         """
-        if not KeyUtils.IsPublicUncompressed(pub_key_bytes):
-            raise ValueError("Public uncompressed key is required for Ethereum address")
+        pub_key_obj = AddrUtils.ValidateAndGetSecp256k1Key(pub_key)
 
-        addr = sha3.keccak_256(pub_key_bytes).hexdigest()[EthAddrConst.START_BYTE:]
-        return EthAddrConst.PREFIX + EthAddrUtils.ChecksumEncode(addr)
+        # First byte of the uncompressed key (i.e. 0x04) is not needed
+        kekkak_hex = ConvUtils.BytesToHexString(CryptoUtils.Kekkak256(pub_key_obj.RawUncompressed().ToBytes()[1:]))
+        addr = kekkak_hex[EthAddrConst.START_BYTE:]
+        return EthereumConf.ADDR_PREFIX + EthAddrUtils.ChecksumEncode(addr)
