@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,37 +20,38 @@
 
 
 # Imports
-import binascii
+from typing import Any, Union
+from bip_utils.addr.iaddr_encoder import IAddrEncoder
 from bip_utils.addr.eth_addr import EthAddr
 from bip_utils.base58 import Base58Encoder
+from bip_utils.coin_conf import TronConf
+from bip_utils.ecc import IPublicKey
+from bip_utils.utils.misc import ConvUtils
 
 
-class TrxAddrConst:
-    """ Class container for Tron address constants. """
-
-    # Prefix
-    PREFIX: str = "41"
-
-
-class TrxAddr:
+class TrxAddr(IAddrEncoder):
     """ Tron address class. It allows the Tron address generation. """
 
     @staticmethod
-    def ToAddress(pub_key_bytes: bytes) -> str:
+    def EncodeKey(pub_key: Union[bytes, IPublicKey],
+                  **kwargs: Any) -> str:
         """ Get address in Tron format.
+            **kwargs: Not used
 
         Args:
-            pub_key_bytes (bytes): Public key bytes
+            pub_key (bytes or IPublicKey): Public key bytes or object
+            **kwargs: Not used
 
         Returns:
             str: Address string
 
         Raised:
-            ValueError: If the key is not a public uncompressed key
+            ValueError: If the public key is not valid
+            TypeError: If the public key is not secp256k1
         """
 
         # Get address in Ethereum format (remove "0x" at the beginning)
-        addr = EthAddr.ToAddress(pub_key_bytes)[2:]
+        eth_addr = EthAddr.EncodeKey(pub_key)[2:]
 
         # Add prefix and encode
-        return Base58Encoder.CheckEncode(binascii.unhexlify(TrxAddrConst.PREFIX + addr))
+        return Base58Encoder.CheckEncode(TronConf.ADDR_PREFIX + ConvUtils.HexStringToBytes(eth_addr))
