@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,27 +20,35 @@
 
 
 # Imports
-from bip_utils.addr.P2PKH_addr import P2PKH
+from typing import Any, Union
+from bip_utils.addr.iaddr_encoder import IAddrEncoder
+from bip_utils.addr.P2PKH_addr import P2PKHAddr
 from bip_utils.base58 import Base58Alphabets
-from bip_utils.conf import RippleConf
+from bip_utils.coin_conf import RippleConf
+from bip_utils.ecc import IPublicKey
 
 
-class XrpAddr:
+class XrpAddr(IAddrEncoder):
     """ Ripple address class. It allows the Ripple address generation. """
 
     @staticmethod
-    def ToAddress(pub_key_bytes: bytes) -> str:
+    def EncodeKey(pub_key: Union[bytes, IPublicKey],
+                  **kwargs: Any) -> str:
         """ Get address in Ripple format.
 
         Args:
-            pub_key_bytes (bytes): Public key bytes
+            pub_key (bytes or IPublicKey): Public key bytes or object
+            **kwargs: Not used
 
         Returns:
             str: Address string
 
         Raises:
-            ValueError: If key is not a public compressed key
+            ValueError: If the public key is not valid
+            TypeError: If the public key is not secp256k1
         """
 
         # Ripple address is just a P2PKH address with a different Base58 alphabet
-        return P2PKH.ToAddress(pub_key_bytes, RippleConf.P2PKH_NET_VER.Main(), Base58Alphabets.RIPPLE)
+        return P2PKHAddr.EncodeKey(pub_key,
+                                   net_ver=RippleConf.P2PKH_NET_VER,
+                                   base58_alph=Base58Alphabets.RIPPLE)
