@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,15 +23,15 @@
 from enum import Enum, auto, unique
 from typing import Dict
 from bip_utils.base58.base58_ex import Base58ChecksumError
-from bip_utils.utils import ConvUtils, CryptoUtils
+from bip_utils.utils.misc import ConvUtils, CryptoUtils
 
 
 @unique
 class Base58Alphabets(Enum):
     """ Enumerative for Base58 alphabet. """
 
-    BITCOIN = auto(),
-    RIPPLE = auto(),
+    BITCOIN = auto()
+    RIPPLE = auto()
 
 
 class Base58Const:
@@ -77,7 +77,7 @@ class Base58Encoder:
             alph_idx (Base58Alphabets, optional): Alphabet index, Bitcoin by default
 
         Returns:
-            str: Base58 encoded string
+            str: Encoded string
 
         Raises:
             TypeError: If alphabet index is not a Base58Alphabets enumerative
@@ -113,7 +113,7 @@ class Base58Encoder:
             alph_idx (Base58Alphabets, optional): Alphabet index, Bitcoin by default
 
         Returns:
-            str: Base58 encoded string with checksum
+            str: Encoded string with checksum
 
         Raises:
             TypeError: If alphabet index is not a Base58Alphabets enumerative
@@ -132,11 +132,11 @@ class Base58Decoder:
         """ Decode bytes from a Base58 string.
 
         Args:
-            data_str (string)                   : Data string
+            data_str (str)                      : Data string
             alph_idx (Base58Alphabets, optional): Alphabet index, Bitcoin by default
 
         Returns:
-            bytes: Base58 decoded bytes
+            bytes: Decoded bytes
 
         Raises:
             TypeError: If alphabet index is not a Base58Alphabets enumerative
@@ -154,7 +154,7 @@ class Base58Decoder:
 
         dec = bytearray()
         while val > 0:
-            val, mod = divmod(val, 256)
+            val, mod = divmod(val, 2**8)
             dec.append(mod)
 
         # Get padding length
@@ -172,7 +172,7 @@ class Base58Decoder:
             alph_idx (Base58Alphabets, optional): Alphabet index, Bitcoin by default
 
         Returns:
-            bytes: Base58 decoded bytes (checksum removed)
+            bytes: Decoded bytes (checksum removed)
 
         Raises:
             ValueError: If the string is not a valid Base58 format
@@ -191,6 +191,9 @@ class Base58Decoder:
 
         # Verify checksum
         if checksum_bytes != comp_checksum:
-            raise Base58ChecksumError("Invalid checksum (expected %s, got %s)" % (comp_checksum, checksum_bytes))
+            raise Base58ChecksumError(
+                f"Invalid checksum (expected {ConvUtils.BytesToHexString(comp_checksum)}, "
+                f"got {ConvUtils.BytesToHexString(checksum_bytes)})"
+            )
 
         return data_bytes
