@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -18,14 +18,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-# Specifications:
-# https://github.com/bitcoincashorg/bitcoincash.org/blob/master/spec/cashaddr.md
+# Reference: https://github.com/bitcoincashorg/bitcoincash.org/blob/master/spec/cashaddr.md
 
 # Imports
 from typing import List, Tuple
 from bip_utils.bech32.bech32_base import Bech32DecoderBase, Bech32EncoderBase, Bech32BaseUtils
 from bip_utils.bech32.bech32_ex import Bech32FormatError
-from bip_utils.utils import ConvUtils
+from bip_utils.utils.misc import ConvUtils
 
 
 class BchBech32Const:
@@ -33,12 +32,12 @@ class BchBech32Const:
 
     # Separator
     SEPARATOR: str = ":"
-    # Checkum length
-    CHECKSUM_LEN: int = 8
-    # Minimum data length
-    DATA_MIN_LEN: int = 2
-    # Maximum data length
-    DATA_MAX_LEN: int = 40
+    # Checkum length in bytes
+    CHECKSUM_BYTE_LEN: int = 8
+    # Minimum data length in bytes
+    DATA_MIN_BYTE_LEN: int = 2
+    # Maximum data length in bytes
+    DATA_MAX_BYTE_LEN: int = 40
 
 
 class BchBech32Utils:
@@ -102,7 +101,7 @@ class BchBech32Utils:
 
         values = BchBech32Utils.HrpExpand(hrp) + data
         polymod = BchBech32Utils.PolyMod(values + [0, 0, 0, 0, 0, 0, 0, 0])
-        return [(polymod >> 5 * (7 - i)) & 0x1f for i in range(BchBech32Const.CHECKSUM_LEN)]
+        return [(polymod >> 5 * (7 - i)) & 0x1f for i in range(BchBech32Const.CHECKSUM_BYTE_LEN)]
 
     @staticmethod
     def VerifyChecksum(hrp: str,
@@ -180,18 +179,19 @@ class BchBech32Decoder(Bech32DecoderBase):
         """
 
         # Decode string
-        hrpgot, data = BchBech32Decoder._DecodeBech32(addr, BchBech32Const.SEPARATOR, BchBech32Const.CHECKSUM_LEN)
+        hrpgot, data = BchBech32Decoder._DecodeBech32(addr, BchBech32Const.SEPARATOR, BchBech32Const.CHECKSUM_BYTE_LEN)
 
         # Check HRP
         if hrpgot != hrp:
-            raise Bech32FormatError("Invalid format (HRP not valid, expected %s, got %s)" % (hrp, hrpgot))
+            raise Bech32FormatError(f"Invalid format (HRP not valid, expected {hrp}, got {hrpgot})")
 
         # Convert back from base32
         conv_data = Bech32BaseUtils.ConvertFromBase32(data)
 
         # Check converted data
-        if len(conv_data) < BchBech32Const.DATA_MIN_LEN or len(conv_data) > BchBech32Const.DATA_MAX_LEN:
-            raise Bech32FormatError("Invalid format (length not valid)")
+        if (len(conv_data) < BchBech32Const.DATA_MIN_BYTE_LEN or
+                len(conv_data) > BchBech32Const.DATA_MAX_BYTE_LEN):
+            raise Bech32FormatError(f"Invalid format (length not valid: {len(conv_data)})")
 
         return conv_data[0], ConvUtils.ListToBytes(conv_data[1:])
 
