@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Emanuele Bellocchia
+# Copyright (c) 2021 Emanuele Bellocchia
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,9 +20,11 @@
 
 
 # Imports
+from typing import Union
 from bip_utils.base58 import Base58Decoder, Base58Encoder
-from bip_utils.conf import BitcoinConf
-from bip_utils.utils import ConvUtils, KeyUtils
+from bip_utils.coin_conf import BitcoinConf
+from bip_utils.ecc import IPrivateKey, Secp256k1PrivateKey
+from bip_utils.utils.misc import ConvUtils
 
 
 class WifConst:
@@ -36,15 +38,16 @@ class WifEncoder:
     """ WIF encoder class. It provides methods for encoding to WIF format. """
 
     @staticmethod
-    def Encode(key_bytes: bytes,
-               compr_pub_key: bool = True,
-               net_addr_ver: bytes = BitcoinConf.WIF_NET_VER.Main()) -> str:
+    def Encode(priv_key: Union[bytes, IPrivateKey],
+               net_ver: bytes = BitcoinConf.WIF_NET_VER_MN,
+               compr_pub_key: bool = True) -> str:
         """ Encode key bytes into a WIF string.
 
         Args:
-            key_bytes (bytes)             : Key bytes
-            compr_pub_key (bool)          : True if private key corresponds to a compressed public key, false otherwise
-            net_addr_ver (bytes, optional): Net address version, default is Bitcoin main network
+            priv_key (bytes or IPrivateKey): Private key bytes or object
+            net_ver (bytes, optional)      : Net version (Bitcoin main net by default)
+            compr_pub_key (bools, optional): True if private key corresponds to a compressed public key,
+                                             false otherwise
 
         Returns:
             str: WIF encoded string
@@ -53,19 +56,23 @@ class WifEncoder:
             ValueError: If the key is not valid
         """
 
-        # Check key
-        if not KeyUtils.IsPrivate(key_bytes):
-            raise ValueError("Invalid key (%s)" % ConvUtils.BytesToHexString(key_bytes))
+        # Convert to private key to check if bytes are valid
+        if isinstance(priv_key, bytes):
+            priv_key = Secp256k1PrivateKey.FromBytes(priv_key)
+        elif not isinstance(priv_key, Secp256k1PrivateKey):
+            raise TypeError("A secp256k1 private key is required")
+
+        priv_key = priv_key.Raw().ToBytes()
 
         # Add suffix if correspond to a compressed public key
         if compr_pub_key:
-            key_bytes += WifConst.COMPR_PUB_KEY_SUFFIX
+            priv_key += WifConst.COMPR_PUB_KEY_SUFFIX
 
         # Add net address version
-        key_bytes = net_addr_ver + key_bytes
+        priv_key = net_ver + priv_key
 
         # Encode key
-        return Base58Encoder.CheckEncode(key_bytes)
+        return Base58Encoder.CheckEncode(priv_key)
 
 
 class WifDecoder:
@@ -73,12 +80,12 @@ class WifDecoder:
 
     @staticmethod
     def Decode(wif_str: str,
-               net_addr_ver: bytes = BitcoinConf.WIF_NET_VER.Main()) -> bytes:
+               net_ver: bytes = BitcoinConf.WIF_NET_VER_MN) -> bytes:
         """ Decode key bytes from a WIF string.
 
         Args:
-            wif_str (str)                 : WIF string
-            net_addr_ver (bytes, optional): Net address version, default is Bitcoin main network
+            wif_str (str)            : WIF string
+            net_ver (bytes, optional): Net version (Bitcoin main net by default)
 
         Returns:
             bytes: Key bytes
@@ -92,23 +99,25 @@ class WifDecoder:
         key_bytes = Base58Decoder.CheckDecode(wif_str)
 
         # Check net version
-        if key_bytes[0] != ord(net_addr_ver):
-            raise ValueError("Invalid net version (expected %x, got %x)" % (ord(net_addr_ver), key_bytes[0]))
+        if key_bytes[0] != ord(net_ver):
+            raise ValueError(
+                f"Invalid net version (expected 0x{ord(net_ver):02X}, got 0x{key_bytes[0]:02X})"
+            )
 
         # Remove net version
         key_bytes = key_bytes[1:]
 
         # Remove suffix if correspond to a compressed public key
-        if KeyUtils.IsPrivate(key_bytes[:-1]):
+        if Secp256k1PrivateKey.IsValidBytes(key_bytes[:-1]):
             # Check the compressed public key suffix
             if key_bytes[-1] != ord(WifConst.COMPR_PUB_KEY_SUFFIX):
-                raise ValueError("Invalid compressed public key suffix (expected %x, got %x)" %
-                                 (ord(WifConst.COMPR_PUB_KEY_SUFFIX), key_bytes[-1]))
+                raise ValueError(
+                    f"Invalid compressed public key suffix (expected 0x{ord(WifConst.COMPR_PUB_KEY_SUFFIX):02X}, "
+                    f"got 0x{key_bytes[-1]:02X})"
+                )
             # Remove it
             key_bytes = key_bytes[:-1]
-
-        # Check if valid
-        if not KeyUtils.IsValid(key_bytes):
-            raise ValueError("Invalid decoded key (%s)" % ConvUtils.BytesToHexString(key_bytes))
+        elif not Secp256k1PrivateKey.IsValidBytes(key_bytes):
+            raise ValueError(f"Invalid decoded key ({ConvUtils.BytesToHexString(key_bytes)})")
 
         return key_bytes
